"use strict";
// Copyright (c) Martin Renou
// Distributed under the terms of the Modified BSD License.
Object.defineProperty(exports, "__esModule", { value: true });
const base_1 = require("@jupyter-widgets/base");
const version_1 = require("./version");
class CanvasModel extends base_1.DOMWidgetModel {
    defaults() {
        return Object.assign({}, super.defaults(), { _model_name: CanvasModel.model_name, _model_module: CanvasModel.model_module, _model_module_version: CanvasModel.model_module_version, _view_name: CanvasModel.view_name, _view_module: CanvasModel.view_module, _view_module_version: CanvasModel.view_module_version, size: [], fill_style: 'black', stroke_style: 'black' });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        this.commands_cache = [];
        this.cache_set_command('fill_style', 'fillStyle');
        this.cache_set_command('stroke_style', 'strokeStyle');
        this.cache_set_command('global_alpha', 'globalAlpha');
        this.on('msg:custom', (command) => { this.commands_cache.push(command); });
        this.on('change:fill_style', () => { this.cache_set_command('fill_style', 'fillStyle'); });
        this.on('change:stroke_style', () => { this.cache_set_command('stroke_style', 'strokeStyle'); });
        this.on('change:global_alpha', () => { this.cache_set_command('global_alpha', 'globalAlpha'); });
    }
    cache_set_command(python_name, ts_name) {
        this.commands_cache.push({
            name: 'set',
            attr: ts_name,
            value: this.get(python_name)
        });
    }
}
CanvasModel.serializers = Object.assign({}, base_1.DOMWidgetModel.serializers);
CanvasModel.model_name = 'CanvasModel';
CanvasModel.model_module = version_1.MODULE_NAME;
CanvasModel.model_module_version = version_1.MODULE_VERSION;
CanvasModel.view_name = 'CanvasView';
CanvasModel.view_module = version_1.MODULE_NAME;
CanvasModel.view_module_version = version_1.MODULE_VERSION;
exports.CanvasModel = CanvasModel;
class CanvasView extends base_1.DOMWidgetView {
    render() {
        this.canvas = document.createElement('canvas');
        this.canvas.width = '100%';
        this.canvas.height = '100%';
        this.el.appendChild(this.canvas);
        this.el.height = '500px';
        this.el.overflow = 'hidden';
        this.el.flex = '1 1 auto';
        this.ctx = this.canvas.getContext('2d');
        this.resize_canvas();
        this.first_draw();
        this.model_events();
    }
    first_draw() {
        // Replay all the commands that were received until this view was created
        for (const command of this.model.commands_cache) {
            if (command.name == 'set') {
                this.ctx[command.attr] = command.value;
            }
            else {
                this.ctx[command.name](...command.args);
            }
        }
    }
    model_events() {
        this.model.on('msg:custom', (command) => {
            this.ctx[command.name](...command.args);
        });
        this.model.on('change:size', () => { this.resize_canvas(); });
        this.model.on('change:fill_style', () => { this.ctx.fillStyle = this.model.get('fill_style'); });
        this.model.on('change:stroke_style', () => { this.ctx.strokeStyle = this.model.get('stroke_style'); });
        this.model.on('change:global_alpha', () => { this.ctx.globalAlpha = this.model.get('global_alpha'); });
    }
    resize_canvas() {
        const size = this.model.get('size');
        this.canvas.setAttribute('width', size[0]);
        this.canvas.setAttribute('height', size[1]);
    }
}
exports.CanvasView = CanvasView;
//# sourceMappingURL=widget.js.map